@@ -6,15 +6,18 @@ from __future__ import print_function
 from __future__ import unicode_literals
 from collections import namedtuple
 from platform import python_version_tuple
+import re
 
 
 if python_version_tuple()[0] < "3":
+    from itertools import izip_longest
     _none_type = type(None)
     _int_type = int
     _float_type = float
     _text_type = unicode
     _binary_type = str
 else:
+    from itertools import zip_longest as izip_longest
     from functools import reduce
     _none_type = type(None)
     _int_type = int
@@ -24,7 +27,7 @@ else:
 
 
 __all__ = ["tabulate"]
-__version__ = "0.4.4"
+__version__ = "0.5"
 
 
 Line = namedtuple("Line", ["begin", "hline", "sep", "end"])
@@ -100,6 +103,10 @@ _table_formats = {"simple":
                               with_header_hide=[],
                               without_header_hide=["linebelowheader"])}
 
+
+_invisible_codes = re.compile("\x1b\[\d*m")  # ANSI color codes
+
+
 def simple_separated_format(separator):
     """Construct a simple TableFormat with columns separated by a separator.
 
@@ -153,8 +160,16 @@ def _type(string):
     True
     >>> _type("1") is type(1)
     True
+    >>> _type(u'\x1b[31m42\x1b[0m') is type(42)
+    True
+    >>> _type('\x1b[31m42\x1b[0m') is type(42)
+    True
 
     """
+
+    if isinstance(string, _text_type) or isinstance(string, _binary_type):
+        string = _strip_invisible(string)
+
     if string is None:
         return _none_type
     elif _isint(string):
@@ -201,7 +216,8 @@ def _padleft(width, s):
     True
 
     """
-    fmt = u"{0:>%ds}" % width
+    iwidth = width + len(s) - len(_strip_invisible(s))
+    fmt = u"{0:>%ds}" % iwidth
     return fmt.format(s)
 
 
@@ -212,7 +228,8 @@ def _padright(width, s):
     True
 
     """
-    fmt = u"{0:<%ds}" % width
+    iwidth = width + len(s) - len(_strip_invisible(s))
+    fmt = u"{0:<%ds}" % iwidth
     return fmt.format(s)
 
 
@@ -223,10 +240,29 @@ def _padboth(width, s):
     True
 
     """
-    fmt = u"{0:^%ds}" % width
+    iwidth = width + len(s) - len(_strip_invisible(s))
+    fmt = u"{0:^%ds}" % iwidth
     return fmt.format(s)
 
 
+def _strip_invisible(s):
+    "Remove invisible ANSI color codes."
+    return re.sub(_invisible_codes, "", s)
+
+
+def _visible_width(s):
+    """Visible width of a printed string. ANSI color codes are removed.
+
+    >>> _visible_width('\x1b[31mhello\x1b[0m'), _visible_width("world")
+    (5, 5)
+
+    """
+    if isinstance(s, _text_type) or isinstance(s, _binary_type):
+        return len(_strip_invisible(s))
+    else:
+        return len(_text_type(s))
+
+
 def _align_column(strings, alignment, minwidth=0):
     """[string] -> [padded_string]
 
@@ -249,8 +285,9 @@ def _align_column(strings, alignment, minwidth=0):
     else:
         strings = [s.strip() for s in strings]
         padfn = _padright
-    maxwidth = max(max(map(len, strings)), minwidth)
-    return [padfn(maxwidth, s) for s in strings]
+    maxwidth = max(max(map(_visible_width, strings)), minwidth)
+    padded_strings = [padfn(maxwidth, s) for s in strings]
+    return padded_strings
 
 
 def _more_generic(type1, type2):
@@ -313,7 +350,63 @@ def _align_header(header, alignment, width):
         return _padleft(width, header)
 
 
-def tabulate(list_of_lists, headers=[], tablefmt="simple",
+def _normalize_tabular_data(tabular_data, headers):
+    """Transform a supported data type to a list of lists, and a list of headers.
+
+    Supported tabular data types:
+
+    * list-of-lists or another iterable of iterables
+
+    * 2D NumPy arrays
+
+    * dict of iterables (usually used with headers="keys")
+
+    * pandas.DataFrame (usually used with headers="keys")
+
+    The first row can be used as headers if headers="firstrow",
+    column indices can be used as headers if headers="keys".
+
+    """
+
+    if hasattr(tabular_data, "keys") and hasattr(tabular_data, "values"):
+        # dict-like and pandas.DataFrame?
+        if hasattr(tabular_data.values, "__call__"):
+            # likely a conventional dict
+            keys = tabular_data.keys()
+            rows = list(izip_longest(*tabular_data.values()))  # columns have to be transposed
+        elif hasattr(tabular_data, "index"):
+            # values is a property, has .index => it's likely a pandas.DataFrame (pandas 0.11.0)
+            keys = tabular_data.keys()
+            vals = tabular_data.values  # values matrix doesn't need to be transposed
+            names = tabular_data.index
+            rows = [[v]+list(row) for v,row in zip(names, vals)]
+        else:
+            raise ValueError("tabular data doesn't appear to be a dict or a DataFrame")
+
+        if headers == "keys":
+            headers = list(map(_text_type,keys))  # headers should be strings
+
+    else:  # it's a usual an iterable of iterables, or a NumPy array
+        rows = tabular_data
+
+        if headers == "keys" and len(rows) > 0:  # keys are column indices
+            headers = list(map(_text_type, range(len(rows[0]))))
+
+    # take headers from the first row if necessary
+    if headers == "firstrow" and len(rows) > 0:
+        headers = list(map(_text_type, rows[0])) # headers should be strings
+        rows = list(rows[1:])
+
+    # pad with empty headers for initial columns if necessary
+    if headers and len(rows) > 0:
+       nhs = len(headers)
+       ncols = len(rows[0])
+       headers = [u""]*(ncols - nhs) + list(headers)
+
+    return rows, headers
+
+
+def tabulate(tabular_data, headers=[], tablefmt="simple",
              floatfmt="g", numalign="decimal", stralign="left",
              missingval=u""):
     """Format a fixed width table for pretty printing.
@@ -325,8 +418,36 @@ def tabulate(list_of_lists, headers=[], tablefmt="simple",
       2  10001
     ---  ---------
 
-    If headers is not empty, it is used as a list of column names
-    to print a nice header. Otherwise a headerless table is produced.
+    The first required argument (`tabular_data`) can be a
+    list-of-lists (or another iterable or iterables), a dictionary of
+    iterables, a two-dimensional NumPy array, or a Pandas' dataframe.
+
+
+    Table headers
+    -------------
+
+    To print nice column headers, supply the second argument (`headers`):
+
+      - `headers` can be an explicit list of column headers
+      - if `headers="firstrow"`, then the first row of data is used
+      - if `headers="keys"`, then dictionary keys or column indices are used
+
+    Otherwise a headerless table is produced.
+
+    If the number of headers is less than the number of columns, they
+    are supposed to be names of the last columns. This is consistent
+    with the plain-text format of R and Pandas' dataframes.
+
+    >>> print(tabulate([["sex","age"],["Alice","F",24],["Bob","M",19]],
+    ...       headers="firstrow"))
+           sex      age
+    -----  -----  -----
+    Alice  F         24
+    Bob    M         19
+
+
+    Column alignment
+    ----------------
 
     `tabulate` tries to detect column types automatically, and aligns
     the values properly. By default it aligns decimal points of the
@@ -335,6 +456,10 @@ def tabulate(list_of_lists, headers=[], tablefmt="simple",
     (`numalign`, `stralign`) are: right, center, left, decimal (only
     for `numalign`).
 
+
+    Table formats
+    -------------
+
     `floatfmt` is a format specification used for columns which
     contain numeric data with a decimal point.
 
@@ -451,6 +576,9 @@ def tabulate(list_of_lists, headers=[], tablefmt="simple",
     ====  ========
 
     """
+
+    list_of_lists, headers = _normalize_tabular_data(tabular_data, headers)
+
     # format rows and columns, convert numeric values to strings
     cols = list(zip(*list_of_lists))
     coltypes = list(map(_column_type, cols))
@@ -459,18 +587,18 @@ def tabulate(list_of_lists, headers=[], tablefmt="simple",
 
     # align columns
     aligns = [numalign if ct in [int,float] else stralign for ct in coltypes]
-    minwidths = [len(h) + 2 for h in headers] if headers else [0]*len(cols)
+    minwidths = [_visible_width(h)+2 for h in headers] if headers else [0]*len(cols)
     cols = [_align_column(c, a, minw)
             for c, a, minw in zip(cols, aligns, minwidths)]
 
     if headers:
         # align headers and add headers
-        minwidths = [max(minw, len(c[0])) for minw, c in zip(minwidths, cols)]
+        minwidths = [max(minw, _visible_width(c[0])) for minw, c in zip(minwidths, cols)]
         headers = [_align_header(h, a, minw)
                    for h, a, minw in zip(headers, aligns, minwidths)]
         rows = list(zip(*cols))
     else:
-        minwidths = [len(c[0]) for c in cols]
+        minwidths = [_visible_width(c[0]) for c in cols]
         rows = list(zip(*cols))
 
     if not isinstance(tablefmt, TableFormat):
